# SafeSpawn release 1 by David Bowland
# ./addons/eventscripts/safespawn/safespawn.py

# >>> To configure this addon please see safespawn.cfg <<<

"""
Provides protection from damage after spawn. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib


info = es.AddonInfo()
info.name = 'SafeSpawn'
info.version = '1'
info.url = 'http://addons.eventscripts.com/addons/view/safespawn'
info.basename = 'safespawn'
info.author = 'SuperDave'


dict_players = {}

dict_options = {'safespawn_protection_time':[6, 'Number of seconds after spawning players will be invincible (should be greater than or equal to 1)'], 'safespawn_protection_health':[0, 'Amount of health invincible players will appear to have (up to 511)--set to 0 for no health display'], 'safespawn_protection_slay':[0, '0 = no change, 1 = slay players who attack an invincible player'], 'safespawn_protection_loseondamage':[1, '0 = players will be invicible for the entire protection time, 1 = invincible players will lose invincibility after causing damage to another player'], 'safespawn_fade_alpha':[100, 'Alpha value to give players when they become invincible'], 'safespawn_fade_steps':[4, 'Number of fading steps before players become fully visible'], 'safespawn_announce_start_console':[0, '0 = no change, 1 = announce to the player in the console when he or she becomes invincible, 2 = announce to all players in the console when a player becomes invincible'], 'safespawn_announce_start_chat':[0, '0 = no change, 1 = announce to the player in the chat area when he or she becomes invincible, 2 = announce to all players in the chat area when a player becomes invincible'], 'safespawn_announce_start_center':[1, '0 = no change, 1 = announce to the player in the center when he or she becomes invincible, 2 = announce to all players in the center when a player becomes invincible'], 'safespawn_announce_end_console':[0, '0 = no change, 1 = announce to the player in the console when he or she is no longer invincible, 2 = announce to all players in the console when a player is no longer invincible'], 'safespawn_announce_end_chat':[0 ,'0 = no change, 1 = announce to the player in the chat area when he or she is no longer invincible, 2 = announce to all players in the chat area when a player is no longer invincible'], 'safespawn_announce_end_center':[1, '0 = no change, 1 = announce to the player in the center when he or she is no longer invincible, 2 = announce to all players in the center when a player is no longer invincible'], 'safespawn_announce_slay_console':[0, '0 = no change, 1 = announce to the player in the console when he or she is slain for attacking an invincible player, 2 = announce to all players in the console a player is slain for attacking an invincible player'], 'safespawn_announce_slay_chat':[2, '0 = no change, 1 = announce to the player in the chat area when he or she is slain for attacking an invincible player, 2 = announce to all players in the chat when a player is slain for attacking an invincible player'], 'safespawn_announce_slay_center':[1, '0 = no change, 1 = announce to the player in the center when he or she is slain for attacking an invincible player, 2 = announce to all players in the center a player is slain for attacking an invincible player']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   """
   global func_lang_text

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('safespawn') + '/safespawn.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/safespawn/safespawn.cfg')
   else:
      es.dbgmsg(0, 'SafeSpawn: Unable to load safespawn.cfg! Please ensure it is in the ./safespawn/ directory.')

   str_lang_path = es.getAddonPath('safespawn') + '/safespawn_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'SafeSpawn: Unable to load safespawn_languages.ini! Please ensure it is in the ./safespawn/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang


def player_spawn(event_var):
   """
   Cancels outstanding delays for the spawning player
   Sets player health and alpha to beginning invincible values
   Calculates the time between alpha steps and stores that into float_interval
   Starts the spawn_step delay loop
   """
   global dict_players

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      gamethread.cancelDelayed('safespawn_%s' % str_userid)

   es.server.cmd('es_xfire %s !self addoutput \"rendermode 1"' % str_userid)
   int_fade_alpha = int(es.ServerVar('safespawn_fade_alpha'))
   if int_fade_alpha <> 255:
      es.server.cmd('es_xfire %s !self alpha %s' % (str_userid, int_fade_alpha))

   int_steps = int(es.ServerVar('safespawn_fade_steps'))
   float_protection_time = float(es.ServerVar('safespawn_protection_time'))
   if int_steps:
      float_interval = float_protection_time / float(int_steps)
   else:
      float_interval = float_protection_time

   display_message(str_userid, 'start', {'time':float_protection_time})
   dict_players[str_userid] = {'interval':float_interval, 'health':int(event_var['es_userhealth']), 'armor':int(event_var['es_userarmor']), 'protection_health':1024 + es.ServerVar('safespawn_protection_health'), 'step':-1, 'total_steps':int_steps, 'alpha':int_fade_alpha}
   gamethread.delayedname(0.1, 'safespawn_%s' % str_userid, spawn_step, str_userid)


def player_hurt(event_var):
   """
   Resets player health if the damaged player is invincible
   Slays the attacking player
   """
   global dict_players

   str_userid = event_var['userid']
   str_attacker = event_var['attacker']
   if dict_players.has_key(str_userid):
      es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', dict_players[str_userid]['protection_health'])
      es.setplayerprop(str_userid, 'CCSPlayer.m_ArmorValue', dict_players[str_userid]['armor'])

      if int(es.ServerVar('safespawn_protection_slay')) and int(str_attacker):
         es.cexec(str_attacker, 'kill')
         display_message(str_userid, 'slay', {'player':event_var['es_attackername'], 'victim':event_var['es_username']})
   elif dict_players.has_key(str_attacker) and int(es.ServerVar('safespawn_protection_loseondamage')):
      dict_players[str_attacker]['steps'] = dict_players[str_attacker]['total_steps']
      gamethread.cancelDelayed('safespawn_%s' % str_attacker)
      spawn_step(str_attacker)


def player_death(event_var):
   """Calls player_disconnect"""
   player_disconnect(event_var)


def player_disconnect(event_var):
   """Removes the player from the dictionary and stops the delay"""
   global dict_players

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      gamethread.cancelDelayed('safespawn_%s' % str_userid)
      del dict_players[str_userid]


def unload():
   """Resets each player's alpha to normal value and cancels outstanding spawn_step delays"""
   int_fade_alpha = int(es.ServerVar('safespawn_fade_alpha'))
   for str_userid in dict_players:
      gamethread.cancelDelayed('safespawn_%s' % str_userid)
      es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', dict_players[str_userid]['health'])
      if dict_players[str_userid]['alpha'] <> 255:
         es.server.cmd('es_xfire %s !self alpha 255' % str_userid)

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def spawn_step(str_userid):
   """
   Counts a step towards the player losing invinciblity
   Updates player data if the player's health was set by another addon
   Sets player alpha based on step and calls spawn_step again after a delay if necessary
   """
   global dict_players

   int_health = es.getplayerprop(str_userid, 'CBasePlayer.m_iHealth')
   dict_current_player = dict_players[str_userid]
   dict_current_player['step'] += 1

   if int_health <> dict_current_player['protection_health']:
      dict_current_player['health'] = int_health
      es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', dict_current_player['protection_health'])
   int_armor = es.getplayerprop(str_userid, 'CCSPlayer.m_ArmorValue')
   if int_armor <> dict_current_player['armor']:
      dict_current_player['armor'] = int_armor

   if dict_current_player['step'] >= dict_current_player['total_steps']:
      display_message(str_userid, 'end', {'player':es.getplayername(str_userid)})
      es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', dict_current_player['health'])
      es.setplayerprop(str_userid, 'CCSPlayer.m_ArmorValue', dict_current_player['armor'])
      if dict_current_player['alpha'] <> 255:
         es.server.cmd('es_xfire %s !self alpha 255' % str_userid)
      del dict_players[str_userid]
   else:
      if dict_current_player['alpha'] <> 255:
         es.server.cmd('es_xfire %s !self alpha %s' % (str_userid, int(((255 - dict_current_player['alpha']) * dict_current_player['step']) / dict_current_player['total_steps']) + dict_current_player['alpha']))
      gamethread.delayedname(dict_current_player['interval'], 'safespawn_%s' % str_userid, spawn_step, str_userid)


def display_message(str_userid, str_event, dict_tokens):
   """Displays the requisite message in the console, chat area, and center text"""
   str_user_lang = playerlib.getPlayer(str_userid).get('lang')

   int_display = int(es.ServerVar('safespawn_announce_' + str_event  + '_console'))
   if int_display == 2:
      for int_loop_userid in es.getUseridList():
          es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text('invincible ' + str_event + ' all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
   elif int_display:
      es.cexec(str_userid, 'echo ' + remove_tags(func_lang_text('invincible ' + str_event, dict_tokens, str_user_lang)))

   int_display = int(es.ServerVar('safespawn_announce_' + str_event  + '_chat'))
   if int_display == 2:
      for int_loop_userid in es.getUseridList():
          es.tell(int_loop_userid, '#multi', func_lang_text('invincible ' + str_event + ' all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang')))
   elif int_display:
      es.tell(str_userid, '#multi', func_lang_text('invincible ' + str_event, dict_tokens, str_user_lang))

   int_display = int(es.ServerVar('safespawn_announce_' + str_event  + '_center'))
   if int_display == 2:
      for int_loop_userid in es.getUseridList():
          es.centertell(int_loop_userid, remove_tags(func_lang_text('invincible ' + str_event + ' all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))
   elif int_display:
      es.centertell(str_userid, remove_tags(func_lang_text('invincible ' + str_event, dict_tokens, str_user_lang)))


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'SafeSpawn: safespawn_languages.ini not found'